<?php

/**
 * @file
 * Contains CFDefaultFieldComparator.php.
 */

namespace Drupal\changed_fields;

/**
 * Class CFDefaultFieldComparator.
 */
class CFDefaultFieldComparator {

  /**
   * Method that returns comparable properties for existing field type.
   *
   * @param $fieldType
   * @return array
   */
  private function getComparableProperties($fieldType) {
    switch ($fieldType) {
      case 'string':
      case 'string_long':
      case 'text':
      case 'text_long':
      case 'boolean':
      case 'integer':
      case 'float':
      case 'decimal':
      case 'datetime':
      case 'email':
      case 'list_integer':
      case 'list_float':
      case 'list_string':
      case 'telephone':
        $properties = array('value');
        break;

      case 'text_with_summary':
        $properties = array(
          'value',
          'summary',
        );
        break;

      case 'entity_reference':
        $properties = array('target_id');
        break;

      case 'file':
        $properties = array(
          'target_id',
          'description',
        );
        break;

      case 'image':
        $properties = array(
          'fid',
          'width',
          'height',
          'target_id',
          'alt',
          'title'
        );
        break;

      case 'link':
        $properties = array(
          'uri',
          'title',
        );
        break;

      default:
        $properties = $this->getDefaultComparableProperties($fieldType);
        break;
    }

    return $properties;
  }

  /**
   * Method that returns comparable properties for extra or custom field type.
   *
   * Use it if you want to add comparison support
   * for extra or custom field types.
   *
   * @param $fieldType
   * @return array
   */
  protected function getDefaultComparableProperties($fieldType) {
    return array();
  }

  /**
   * Method that compares old and new field values.
   *
   * @param $fieldInfo
   * @param $oldValue
   * @param $newValue
   * @return array|bool
   */
  public function compareFieldValues($fieldInfo, $oldValue, $newValue) {
    $result = TRUE;
    $properties = $this->getComparableProperties($fieldInfo->getType());

    // If value was added or removed then we have already different values.
    if ((!$oldValue && $newValue) || ($oldValue && !$newValue)) {
      $result = $this->makeResultArray($oldValue, $newValue);
    }
    else {
      if ($oldValue && $newValue) {
        // If value was added|removed to|from multi-value field then we have
        // already different values.
        if (count($newValue) != count($oldValue)) {
          $result = $this->makeResultArray($oldValue, $newValue);
        }
        else {
          // Walk through each field value and compare it's properties.
          foreach ($newValue as $key => $value) {
            if (is_array($result)) {
              break;
            }

            foreach ($properties as $property) {
              if ($newValue[$key][$property] != $oldValue[$key][$property]) {
                $result = $this->makeResultArray($oldValue, $newValue);
                break;
              }
            }
          }
        }
      }
    }

    return $result;
  }

  /**
   * Method that generates result array for CFDefaultFieldComparator::compareFieldValues().
   *
   * @param $oldValue
   * @param $newValue
   * @return array
   */
  private function makeResultArray($oldValue, $newValue) {
    return array(
      'old_value' => $oldValue,
      'new_value' => $newValue,
    );
  }

}
