<?php

/**
 * @file
 * Contains CFNodeSubject.php.
 */

namespace Drupal\changed_fields;

use Drupal\node\NodeInterface;

/**
 * Class CFNodeSubject.
 */
class CFNodeSubject implements CFSubjectInterface {

  /**
   * @var NodeInterface
   */
  private $node;

  /**
   * @var array
   */
  private $changedFields;

  /**
   * @var array
   */
  private $info;

  /**
   * @var CFDefaultFieldComparator
   */
  private $fieldComparator;

  /**
   * @var array
   */
  private $observers;

  /**
   * @param NodeInterface $node
   * @param array $info
   * @param CFDefaultFieldComparator $fieldComparator
   */
  public function __construct(NodeInterface $node, array $info, CFDefaultFieldComparator $fieldComparator) {
    $this->node = $node;
    $this->changedFields = array();
    $this->info = $info;
    $this->fieldComparator = $fieldComparator;
  }

  /**
   * {@inheritdoc}
   */
  public function addObserver(CFObserverInterface $observer) {
    $this->observers[$observer->getId()] = $observer;
  }

  /**
   * {@inheritdoc}
   */
  public function removeObserver(CFObserverInterface $observer) {
    if (isset($this->observers[$observer->getId()])) {
      unset($this->observers[$observer->getId()]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function checkNodeFields() {
    foreach ($this->info as $nodeType => $fields) {
      if (!$this->node->isNew() && $this->node->getType() == $nodeType) {
        $changedFields = array();

        foreach ($fields as $fieldName) {
          $oldValue = $this->node->original->get($fieldName)->getValue();
          $newValue = $this->node->get($fieldName)->getValue();
          $fieldInfo = $this->node->get($fieldName)->getFieldDefinition();

          $result = $this->fieldComparator->compareFieldValues($fieldInfo, $oldValue, $newValue);

          if (is_array($result)) {
            $changedFields[$fieldName] = $result;
          }
        }

        if (!empty($changedFields)) {
          $this->changedFields = $changedFields;

          foreach ($this->observers as $observer) {
            $observer->update($this);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getNode() {
    return $this->node;
  }

  /**
   * {@inheritdoc}
   */
  public function getChangedFields() {
    return $this->changedFields;
  }

}
